@@ -22,6 +22,7 @@ Models
 ======
 """
 from functools import partial
+import types
 
 from pymongo import dbref
 
@@ -99,8 +100,38 @@ class MongoBoundDictMixin(object):
     indexes = []
 
     def __hash__(self):
-        "Collection name and id together make the hash."
-        return hash(self.collection) | hash(self.get('_id'))
+        """ Collection name and id together make the hash; document class
+        doesn't matter.
+
+        Raises `TypeError` if collection or id is not set.
+        """
+        if self.collection and self.get_id():
+            return hash(self.collection) | hash(self.get_id())
+        raise TypeError('Document is unhashable: collection or id is not set')
+
+    def __eq__(self, other):
+        # both must inherit to this class
+        if not isinstance(other, MongoBoundDictMixin):
+            return False
+        # both must have collections defined
+        if not self.collection or not other.collection:
+            return False
+        # both must have ids
+        if not self.get_id() or not other.get_id():
+            return False
+
+        # collections must be equal
+        if self.collection != other.collection:
+            return False
+        # ids must be equal
+        if self.get_id() != other.get_id():
+            return False
+
+        return True
+
+    def __ne__(self, other):
+        # this is required to override the call to dict.__eq__()
+        return not self.__eq__(other)
 
     @classmethod
     def _ensure_indexes(cls, db):
@@ -146,9 +177,26 @@ class MongoBoundDictMixin(object):
 
         return object_id
 
+    def get_id(self):
+        return self.get('_id')
+
+    def get_ref(self):
+        _id = self.get_id()
+        if _id is None:
+            return None
+        else:
+            return dbref.DBRef(self.collection, _id)
+
 
 class StructuredDictMixin(object):
     """ A dictionary with structure specification and validation.
+
+    .. attribute:: structure
+
+        The document structure specification. For details see
+        :func:`monk.validation.validate_structure_spec` and
+        :func:`monk.validation.validate_structure`.
+
     """
     structure = {}
     #defaults = {}
@@ -157,7 +205,20 @@ class StructuredDictMixin(object):
     #with_skeleton = True
 
     def _insert_defaults(self):
-        with_defaults = manipulation.merged(self.structure, self)
+        """ Inserts default values from :attr:`StructuredDictMixin.structure`
+        to `self` by merging the two structures
+        (see :func:`monk.manipulation.merged`).
+        """
+        def process_value(value):
+            func_types = types.FunctionType,types.BuiltinFunctionType
+            if isinstance(value, func_types):
+                return value()
+            else:
+                return value
+
+        with_defaults = manipulation.merged(self.structure, self,
+                                            value_processor=process_value)
+
         for key, value in with_defaults.iteritems():
             self[key] = value
 
@@ -177,6 +238,15 @@ class Document(
     ):
     """ A structured dictionary that is bound to MongoDB and supports dot
     notation for access to items.
+
+    Inherits features from:
+
+    * `dict` (builtin),
+    * class:`~monk.modeling.TypedDictReprMixin`,
+    * class:`~monk.modeling.DotExpandedDictMixin`,
+    * class:`~monk.modeling.StructuredDictMixin` and
+    * class:`~monk.modeling.MongoBoundDictMixin`.
+
     """
     def __init__(self, *args, **kwargs):
         super(Document, self).__init__(*args, **kwargs)
